"""Boolean argument."""
from typing import List

from .enum import Argument


class BoolArgument(Argument):
    """Bool Argument."""

    true = ["y", "yes", "t", "true"]
    false = ["n", "no", "f", "false"]

    def add_value(self, value_labels: List[str], value: str) -> None:
        """Recognise and add the value.

        Args:
            value_labels: the name of the lowest-level subargument, split by levels.
                            (highest-argument level first)
            value: the value which should be parsed and saved to the subargument.

        """
        if self.has_value:
            raise ValueError(f"Value for {self._arg_label} is already provided!")

        value = value.lower()

        if value in self.true:
            self._value = True

        elif value in self.false:
            self._value = False

        else:
            self._invalid = True
            raise TypeError(f"Supplied value to argument {self._arg_label} is of incorrect type.")
