"""DateTime Argument Types."""
import datetime
from typing import List

from .argument import Argument


class DateTimeArgument(Argument):
    """Handle datetime creation."""

    def add_value(self, value_labels: List[str], value: str) -> None:
        """Recognise and add the value.

        Args:
            value_labels: the name of the lowest-level subargument, split by levels.
                            (highest-argument level first)
            value: the value which should be parsed and saved to the subargument.

        """
        if self.has_value:
            raise ValueError(f"Value for {self._arg_label} is already provided!")
        try:
            self._value = datetime.datetime.fromisoformat(value)
        except ValueError:
            self._invalid = True
            raise TypeError(f"Supplied value to argument {self._arg_label} is of incorrect format.")


class DateArgument(Argument):
    """Handle date creation."""

    def add_value(self, value_labels: List[str], value: str) -> None:
        """Recognise and add the value.

        Args:
            value_labels: the name of the lowest-level subargument, split by levels.
                            (highest-argument level first)
            value: the value which should be parsed and saved to the subargument.

        """
        if self.has_value:
            raise ValueError(f"Value for {self._arg_label} is already provided!")
        try:
            self._value = datetime.date.fromisoformat(value)
        except ValueError:
            self._invalid = True
            raise TypeError(f"Supplied value to argument {self._arg_label} is of incorrect format.")
