"""The main file, starting the whole program."""
import shlex
from logging.config import dictConfig
from pathlib import Path
from typing import List, Optional

import click

from eppic.app import Application
from eppic.settings import Settings


@click.command()
@click.option('--session', help='Specify session by username. Default is the first in the list.')
@click.option('--input-file', help='Specify the file to get the input from. If not specified, stdin is used.')
@click.option('--config', type=click.Path(exists=True, file_okay=True, dir_okay=False),
              help='Settings file location.')
@click.argument('command', nargs=-1, type=str)
def main(config: Optional[Path] = None, session: Optional[str] = None,
         input_file: Optional[str] = None, command: Optional[List[str]] = None) -> None:
    """Run the client with command-line arguments."""
    settings = Settings(input_file=input_file, settings_file=config, session_id=session)

    if settings.logging:
        dictConfig(settings.logging)

    command_str = " ".join(map(shlex.quote, command)) if command else None

    Application(settings, command_str).run()


if __name__ == "__main__":
    main()
