from typing import Optional, Union
from unittest import TestCase

from eppic.commands.argument_parser import ArgumentParser, _unpack_required_type
from eppic.commands.argument_registry import ArgumentRegistry
from eppic.commands.arguments.argument import Argument


class TestArgumentParser(TestCase):
    """Test ArgumentParser class."""

    def setUp(self):
        self.registry = ArgumentRegistry()
        self.registry.register_argument_by_type(int, Argument)
        self.parser = ArgumentParser({"foo": int, "bar": int, "baz": Optional[int]}, self.registry)

    def test_unpacking_union(self):
        self.assertEqual(_unpack_required_type(Union[None, str, int]), (Union[str, int], False))

    def test_parse_all_labels(self):
        self.parser.parse_arguments([(["foo"], 42), (["bar"], 0)])
        self.assertEqual(
            self.parser.get_parsed_values(),
            {"foo": 42, "bar": 0, "baz": None}
        )

    def test_parse_missing_labels(self):
        self.parser.parse_arguments([(["foo"], 42), (None, 0)])
        self.assertEqual(
            self.parser.get_parsed_values(),
            {"foo": 42, "bar": 0, "baz": None}
        )

    def test_parse_too_many_values(self):
        self.assertRaises(
            ValueError,
            self.parser.parse_arguments, [(["foo"], 42), (["bar"], 0), (["baz"], 0), (None, 3)]
        )

    def test_parse_unknown_label(self):
        self.assertRaises(
            ValueError,
            self.parser.parse_arguments, [(["foo"], 42), (["asdf"], 3)]
        )
